<?php
// Datenbank-Verbindungsparameter
require_once ('config.php');

require_once ('commonutils.php');

class DbUtils {
	private static $timezone = null;
	private static $prefix = null;
	private static $dbname = null;
	
	public static $WORKFLOW_DIGITAL_AND_WORK = 0;
	public static $WORKFLOW_ONLY_DIGITAL = 1;
	public static $WORKFLOW_ONLY_WORK = 2;
	public static $WORKFLOW_WORK_WITH_SERVER = 3;
	
	public static $PRICE_TYPE_BASE = 0;
	public static $PRICE_TYPE_DICOUNT = 1;
	public static $PRICE_TYPE_EXTRA_AMOUNT = 2;
	public static $PROCESSTYPE_BELEG = 1;
	public static $PROCESSTYPE_VORGANG = 2;
	public static $PROCESSTYPE_SONSTIGER_VORGANG = 3;
	public static $OPERATION_IN_BILL_TABLE = 1;
	public static $OPERATION_IN_QUEUE_TABLE = 2;
	public static $OPERATION_IN_CLOSING_TABLE = 3;
	public static $NO_TSE = 0;
	public static $TSE_OK = 1;
	public static $TSE_KNOWN_ERROR = 2;
	public static $TSE_RUNTIME_ERROR = 3;
	public static $TSE_MISCONFIG = 4;
	public static $OSLABEL = "OrderSprinter";
	public static $OSVERSLABEL = "Version";
	
	public static $ORDERTYPE_PRODUCT = 1;
	public static $ORDERTYPE_1ZweckKauf = 2;
	public static $ORDERTYPE_1ZweckEinl = 3;
        
        public static $PAYMENT_BAR = 1;
        public static $PAYMENT_EC = 2;
        public static $PAYMENT_KREDIT = 3;
        public static $PAYMENT_RECHNUNG = 4;
        public static $PAYMENT_UEBERW = 5;
        public static $PAYMENT_LAST = 6;
        public static $PAYMENT_HOTEL = 7;
        public static $PAYMENT_GUEST = 8;

	public static function overruleTimeZone($timezone) {
		self::$timezone = $timezone;
	}
	public static function overrulePrefix($prefix) {
		self::$prefix = $prefix;
	}
	public static function overruleDbName($dbname) {
		self::$dbname = $dbname;
	}
	public static function getDbName() {
		$db = MYSQL_DB;
		if (!is_null(self::$dbname)) {
			$db = self::$dbname;
		}
		return $db;
	}
	
	public static function openDbAndReturnPdoStatic ($doEchoError = true) {
		$dsn = 'mysql:host=' . MYSQL_HOST . ';dbname=' . MYSQL_DB;
		$user = MYSQL_USER;
		$password = MYSQL_PASSWORD;
		$pdo = null;
		try {
			$pdo = new PDO($dsn, $user, $password);
			$pdo ->setAttribute(PDO::ATTR_ERRMODE, PDO::ERRMODE_EXCEPTION);
			$sql = "SET SESSION sql_mode = 'STRICT_TRANS_TABLES,NO_ENGINE_SUBSTITUTION'";
			CommonUtils::execSql($pdo, $sql, null);
		}
		catch (PDOException $e) {
			if ($doEchoError) {
				echo 'Connection failed: ' . $e->getMessage();
			}
		}
		return $pdo;
	}
	function openDbAndReturnPdo () {
		$dsn = 'mysql:host=' . MYSQL_HOST . ';dbname=' . MYSQL_DB;
		$user = MYSQL_USER;
		$password = MYSQL_PASSWORD;
		$pdo = null;
		try {
			$pdo = new PDO($dsn, $user, $password);
			$pdo ->setAttribute(PDO::ATTR_ERRMODE, PDO::ERRMODE_EXCEPTION);
		}
		catch (PDOException $e) {
			echo 'Connection failed: ' . $e->getMessage();
		}
		return $pdo;
	}
	
	function testDbAccess($host,$dbname,$user,$pass) {
		$dsn = 'mysql:host=' . $host . ';dbname=' . $dbname;
		$password = $pass;
		$pdo = null;
		try {
			$pdo = new PDO($dsn, $user, $password);
			$pdo ->setAttribute(PDO::ATTR_ERRMODE, PDO::ERRMODE_EXCEPTION);
		}
		catch (PDOException $e) {
			//
		}
		if ($pdo != null) {
			return true;
		} else {
			return false;
		}
	}
	
	/*
	 * To use sql strings that are easy to read the table names are used
	 * without variables. But since the user can specify a prefix for all
	 * tables the substitution must be done somewhere. This is the function
	 * that replaces the %TABLE% by $prefix_table
	 */
	public static function substTableAlias($sqlString) {
		$prefix = TAB_PREFIX;
		if (!is_null(self::$prefix)) {
			$prefix = self::$prefix;
		}
		return self::substTableAliasCore($sqlString, $prefix);
	}
	
	public static function substTableAliasCore($sqlString,$prefix) {
                $cnv = array(
                    "%queue%" => $prefix . 'queue',
                    "%products%" => $prefix . 'products',
                    "%user%" => $prefix . 'user',
                    "%room%" => $prefix . 'room',
                    "%resttables%" => $prefix . 'resttables',
                    "%bill%"=> $prefix . 'bill',
                    "%customerlog%" => $prefix . 'customerlog',
                    "%customers%" => $prefix . 'customers',
                    "%groups%" => $prefix . 'groups',
                    "%groupcustomer%" => $prefix . 'groupcustomer',
                    "%vacations%" => $prefix . 'vacations',
                    "%tablemaps%" => $prefix . "tablemaps",
                    "%tablepos%" => $prefix . "tablepos",
                    "%pricelevel%" => $prefix . 'pricelevel',
                    "%config%" => $prefix . 'config',
                    "%closing%" => $prefix . 'closing',
                    "%printjobs%" => $prefix . 'printjob',
                    "%hist%" => $prefix . 'hist',
                    "%histprod%" => $prefix . 'histprod',
                    "%histconfig%" => $prefix . 'histconfig',
                    "%histuser%" => $prefix . 'histuser',
                    "%histactions%" => $prefix . 'histactions',
                    "%payment%" => $prefix . 'payment',
                    "%billproducts%" => $prefix . 'billproducts',
                    "%work%" => $prefix . 'work',
                    "%comments%" => $prefix . 'comments',
                    "%hsin%" => $prefix . 'hsin',
                    "%hsout%" => $prefix . 'hsout',
                    "%reservations%" => $prefix . 'reservations',
                    "%logo%" => $prefix . 'logo',
                    "%log%" => $prefix . 'log',
                    "%usedfeatures%" => $prefix . 'usedfeatures',
                    "%performance%" => $prefix . 'performance',
                    "%extras%" => $prefix . 'extras',
                    "%extrasprods%" => $prefix . 'extrasprods',
                    "%queueextras%" => $prefix . 'queueextras',
                    "%ratings%" => $prefix . 'ratings',
                    "%prodimages%" => $prefix . 'prodimages',
                    "%roles%" => $prefix . 'roles',
                    "%recordsqueue%" => $prefix . 'recordsqueue',
                    "%records%" => $prefix . 'records',
                    "%times%" => $prefix . 'times',
                    "%tasks%" => $prefix . 'tasks',
                    "%taskhist%" => $prefix . 'taskhist',
                    "%tsevalues%" => $prefix . 'tsevalues',
                    "%operations%" => $prefix . 'operations',
                    "%terminals%" => $prefix . 'terminals',
                    "%counting%" => $prefix . 'counting',
                    "%vouchers%" => $prefix . 'vouchers',
                    "%orders%" => $prefix . 'orders',
                    "%prodnames%" => $prefix . 'prodnames',
                    "%liveorders%" => $prefix . 'liveorders',
                    "%testchk%" => $prefix . 'testchk',
                    "%prodtype%" => $prefix . 'prodtype'
                );
	
		return strtr($sqlString,$cnv);			
	}
	
	public function resolveTablenamesInSqlString($sqlString) {
		return DbUtils::substTableAlias($sqlString);
	}
	
	public static function getTimeZone() {
		if (is_null(self::$timezone)) {
			if(session_id() == '') {
				session_start();
			}
			if (isset($_SESSION['timezone'])) {
				return $_SESSION['timezone'];
			} else {
				return "Europe/Berlin";
			}
		} else {
			return self::$timezone;
		}
	}
        
	public static function getTimeZoneDb($pdo) {
		if (is_null($pdo)) {
			return "Europe/Berlin";
		}
		try {
			return CommonUtils::getConfigValue($pdo, 'timezone', "Europe/Berlin");
		} catch (Exception $ex) {
			return "Europe/Berlin";
		}
	}
	
        public static $userCols = array(
            array("col" => 'id',                "hist" => 1),
            array("col" => 'username',          "hist" => 1),
            array("col" => 'userpassword',      "hist" => 0),
	    array("col" => 'quickcash',		"hist" => 1),
            array("col" => 'active',            "hist" => 1, "default" => 1),
	    array("col" => 'area',		"hist" => 1, "new" => null	    ,"default" => null,	    "update" => null),	
            array("col" => 'lastmodule',        "hist" => 0, "new" => null	    ,"default" => null,	    "update" => null),
            array("col" => 'ordervolume',       "hist" => 0, "new" => null	    ,"default" => null,	    "update" => null),
            array("col" => 'language',          "hist" => 0, "new" => null,				    "update" => null),
            array("col" => 'receiptprinter',    "hist" => 0, "new" => null	    ,"default" => null,	    "update" => null),
            array("col" => 'roombtnsize',       "hist" => 0, "new" => null	    ,"default" => null,	    "update" => null),
            array("col" => 'tablebtnsize',      "hist" => 0, "new" => null	    ,"default" => null,	    "update" => null),
            array("col" => 'prodbtnsize',       "hist" => 0, "new" => null	    ,"default" => null,	    "update" => null),
            array("col" => 'prefertablemap',    "hist" => 0, "new" => null	    ,"default" => 1,	    "update" => null),
	    array("col" => 'preferimgdesk',     "hist" => 0, "new" => null	    ,"default" => null,	    "update" => null),
	    array("col" => 'preferimgmobile',   "hist" => 0, "new" => null	    ,"default" => null,	    "update" => null),
	    array("col" => 'showplusminus',	"hist" => 0, "new" => null	    ,"default" => null,	    "update" => null),
            array("col" => 'keeptypelevel',     "hist" => 0, "new" => null	    ,"default" => 0,	    "update" => null),
            array("col" => 'extrasapplybtnpos', "hist" => 0, "new" => null	    ,"default" => 1,	    "update" => null)
        );
        
									
        public static $prodCols = array(
            array("col" => 'id',                "hist" => 1,	"property" => "prodid"),
            array("col" => 'shortname',         "hist" => 1,	"property" => "shortName"),
            array("col" => 'longname',          "hist" => 1,	"property" => "longName"),
            array("col" => 'priceA',            "hist" => 1,	"property" => "priceA"),
            array("col" => 'priceB',            "hist" => 1,	"property" => "priceB"),
            array("col" => 'priceC',            "hist" => 1,	"property" => "priceC"),
	    array("col" => 'barcode',		"hist" => 1,	"property" => "barcode"),
	    array("col" => 'unit',		"hist" => 1,	"property" => "unit"),
	    array("col" => 'days',		"hist" => 1,	"property" => "days"),
            array("col" => 'tax',               "hist" => 1,	"property" => "tax"),
	    array("col" => 'togotax',           "hist" => 1,	"property" => "togotax"),
	    array("col" => 'taxaustria',        "hist" => 1,	"property" => "taxaustria"),
	    array("col" => 'amount',	        "hist" => 0,	"property" => "amount"),
            array("col" => 'category',          "hist" => 0,	"property" => "category"),
            array("col" => 'favorite',          "hist" => 1,	"property" => "favorite"),
            array("col" => 'sorting',           "hist" => 1),
            array("col" => 'available',         "hist" => 1,	"property" => "available"),
            array("col" => 'audio',             "hist" => 1,	"property" => "audio"),
	    array("col" => 'prodimageid',       "hist" => 1,	"property" => "prodimageid"),
	    array("col" => 'display',	        "hist" => 1,	"property" => "display"),
            array("col" => 'removed',           "hist" => 0)
        );
	
	private static function dropDBTable($pdo,$tablename) {
		try {
			CommonUtils::execSql($pdo, "DROP TABLE $tablename", null);
			return true;
		} catch (Exception $ex) {
			return false;
		}
	}
	
	public static function checkForInstallUpdateDbRights($pdo) {
		$tableexists = false;
		try {
			$result = CommonUtils::fetchSqlAll($pdo, "SELECT 1 from %testchk% LIMIT 1", null);
			if (count($result) >= 0) {
				$tableexists = true;
			}
		} catch (Exception $ex) {
			$tableexists = false;
		}

		if ($tableexists) {
			$ok = self::dropDBTable($pdo, '%testchk%');
			if (!$ok) {
				return array("status" => "OK","msg" => array("DROP"),"ok" => 0);
			}
		}
		
		try {
			$sql = "CREATE TABLE `%testchk%` (`id` INT (3)) CHARACTER SET utf8 COLLATE utf8_general_ci ENGINE = InnoDb";
			CommonUtils::execSql($pdo, $sql, null);
		} catch (Exception $ex) {
			return array("status" => "OK","msg" => array("CREATE"),"ok" => 0);
		}
		$missingRights = array();
		try {
			CommonUtils::execSql($pdo, "ALTER TABLE %testchk% ADD testfield INT(1) NULL DEFAULT '0' AFTER id", null);
		} catch (Exception $ex) {
			$missingRights[] = "ALTER";
		}
		try {
			CommonUtils::execSql($pdo, "INSERT INTO  %testchk% (id,testfield) VALUES(?,?)", array(1,2));
		} catch (Exception $ex) {
			$missingRights[] = "INSERT";
		}
		try {
			CommonUtils::execSql($pdo, "UPDATE  %testchk% SET testfield=? WHERE id=?", array(10,1));
		} catch (Exception $ex) {
			$missingRights[] = "UPDATE";
		}
		$ok = self::dropDBTable($pdo, '%testchk%');
		if (!$ok) {
			$missingRights[] = "DROP";
		}

		if (count($missingRights) == 0) {
			return array("status" => "OK","msg" => $missingRights,"ok" => 1);
		} else {
			return array("status" => "OK","msg" => $missingRights,"ok" => 0);
		}
	}
}